/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.generator;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;

/**
 * An implementation of OutputStream that writes
 * programming code into a file, with indentation
 * and so on.
 * 
 * @version Régis CLOUARD - Dec 04, 2007
 */
public class FileOutputCode extends PrintStream {
	/** The representation of a tabulation. */
	private final String _tabulation="    ";

	/**
	 * Creates a new print stream.
	 * 
	 * @param out The output stream to which code will be saved.
	 * @throws IOException in case of output errors.
	 */
	public FileOutputCode( File out ) throws IOException {
		super(out);
		out.setExecutable(true,false);
	}

	/**
	 * Prints the string with the specified indentation
	 * before the string.
	 * 
	 * @param s the string to be printed.
	 * @param x the number of white space.
	 */
	public void print( String s, int x ) {
		for (int i =0; i<x; i++)
			super.print(_tabulation);
		super.print(s);
	}

	/**
	 * Prints the string with the specified indentation.
	 * 
	 * @param s the string to be printed.
	 * @param x the number of white space.
	 */
	public void println( String s, int x ) {
		for (int i =0; i<x; i++)
			super.print(_tabulation);
		super.println(s);
	}
}