/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph;

import java.io.File;
import java.util.HashMap;
import java.util.LinkedList;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.graph.item.DataFile;
import fr.greyc.pantheon.ariane.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.graph.item.LoaderItem;
import fr.greyc.pantheon.ariane.graph.item.SaverItem;

/**
 * A <code>ThreadExecution</code> takes into account the
 * execution of a graph item. It allows an asynchronized process.
 *
 * @author Régis CLOUARD - Dec 15, 2011
 */
public final class ThreadDirectoryExecution extends Thread {
	/** The list of items to be executed. */
	private GraphItem[] _currentSelectedItems;
	private LoaderItem _loader;
	private File[] _files;
	private HashMap<SaverItem, File> _saverDirectories;
	private File _loaderFile;

	/**
	 * Creates a graph execution thread.
	 * 
	 * @param selectedItems the list of item to be executed.
	 */
	public ThreadDirectoryExecution( LinkedList<GraphItem> selectedItems, LoaderItem loader, File loaderFile, File[] files, HashMap<SaverItem, File> saverDirectories) {
		// We use a copy of the selectItems since use can modifier the list
		// while the program is executing it.
		_currentSelectedItems = selectedItems.toArray(new GraphItem[selectedItems.size()]);
		_loader = loader;
		_files = files;
		_loaderFile=loaderFile;
		_saverDirectories = saverDirectories;
	}

	/**
	 * Runs the thread.
	 */
	@Override
	public void run( ) {
		Ariane.workspace.setExecution(true);
		for ( File f : _files ) {
			String filename = f.getName();
			int suffixPosition = filename.lastIndexOf('.');
			String basename=filename;
			if (suffixPosition>0) {
				basename = filename.substring(0,suffixPosition);
			}
			_loader.setBinary(f.getAbsolutePath());
			_loader.setTemporaryNickName(DataFile.getFileNameWithoutExtension(f.getName()));

			for ( SaverItem item : _saverDirectories.keySet()) {
				filename = _saverDirectories.get(item).getName();
				String suffix="";
				suffixPosition=filename.lastIndexOf('.');
				if (suffixPosition>0) {
					suffix = filename.substring(suffixPosition);
				}
				item.setBinary(_saverDirectories.get(item).getParent()+File.separator+basename+suffix);
			}
			_loader.resetAll();
			for ( GraphItem g: _currentSelectedItems ) {
				g.execute();
			}
		}
		_currentSelectedItems=null;
		Ariane.workspace.setExecution(false);
		// restore old file names.
		_loader.setBinary(_loaderFile.getAbsolutePath());
		_loader.setTemporaryNickName(DataFile.getFileNameWithoutExtension(_loaderFile.getAbsolutePath()));
		for ( SaverItem item : _saverDirectories.keySet()) {
			item.setBinary(_saverDirectories.get(item).getAbsolutePath());
		}
	}
}