/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph.command;

import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.CannotRedoException;

import fr.greyc.pantheon.ariane.graph.item.CompositeItem;

/**
 * This class is a part of the history manager.
 * Store change on composite status (between editable / uneditable).
 * 
 * @author Régis CLOUARD - Dec 14, 2012
 */
public class ChangeCompositeStatusEdit extends AbstractUndoableEdit {
	private CompositeItem _compositeItem;
	
	private boolean _editable;
	
	/**
	 * Creates a new edit.
	 * @param inSlot the input slot.
	 * @param newValue the new value of the parameter.
	 * @param oldValue the old value of the parameter.
	 */
	public ChangeCompositeStatusEdit( CompositeItem compositeItem, boolean editable ) {
		_compositeItem = compositeItem;
		_editable=editable;
	}

	/**
	 * Re-applies the edit, assuming that it has been undone.
	 */
	@Override
	public final void redo( ) throws CannotRedoException {
		_compositeItem.changeEditableStatus(!_editable);
	}

	/**
	 * Undoes the edit that was made.
	 */
	@Override
	public final void undo( ) throws CannotRedoException {
		_compositeItem.changeEditableStatus(_editable);
	}
}