/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.graph.slot;

import java.awt.Point;
import java.util.ArrayList;

/**
 * A <code>ValueLink</code> is a connection
 * between two slots that supports values.
 * It is represented as a polyline.
 *
 * @author Régis CLOUARD - Jan 16, 2011
 */
public class ValueLink extends Link {
	/**
	 * Creates a new link with the given input slot
	 * as the source.
	 * 
	 * @param slot the input slot that supports this slot.
	 */
	public ValueLink( InSlot slot ) {
		super(slot);
	}

	/**
	 * Builds the polyline that corresponds to the link.
	 *
	 * @return the series of straight lines.
	 */
	@Override
	protected final ArrayList<Point> createPolyline( ) {
		ArrayList<Point> path= new ArrayList<Point>();
		
		// Insert the first point.
		path.add(0,new Point(_targetSlot.getConnectedSlot().getAnchor()));

		int[] xcoord= new int[6];
		int[] ycoord= new int[6];

		xcoord[3] = (int)_targetSlot.getAnchor().getX();
		ycoord[3] = (int)_targetSlot.getAnchor().getY();
		xcoord[0] = (int)(_targetSlot.getConnectedSlot()).getAnchor().getX();
		ycoord[0] = (int)(_targetSlot.getConnectedSlot()).getAnchor().getY();

		if (ycoord[0]<= ycoord[3]) { // nominal case: item 1 below item 2
			ycoord[1] = ycoord[3]-((ycoord[3]-ycoord[0])/2); xcoord[1] = xcoord[0];
			ycoord[2] = ycoord[1]; xcoord[2] = xcoord[3];

			path.add(new Point(xcoord[1],ycoord[1]));
			path.add(new Point(xcoord[2],ycoord[2]));
		} else { //degenerate case... item 2 above item 1
			ycoord[5]= ycoord[3]; xcoord[5]=xcoord[3];
			ycoord[1]= ycoord[0]+10; xcoord[1] = xcoord[0];
			if (xcoord[0] < xcoord[5]-15) { // link in front  of element 1.
				ycoord[2] = ycoord[1]; xcoord[2] = (((xcoord[5]-10) - xcoord[0]) / 2) + xcoord[0];
				ycoord[3] = ycoord[5]-10; xcoord[3]= xcoord[2];
			} else { // link behind item 1.
				ycoord[2] = ycoord[0]+10; xcoord[2] = xcoord[5]+20;
				ycoord[3] = ycoord[5]-10; xcoord[3] = xcoord[2];
			}
			ycoord[4] = ycoord[3]; xcoord[4] = xcoord[5];

			path.add(new Point(xcoord[1],ycoord[1]));
			path.add(new Point(xcoord[2],ycoord[2]));
			path.add(new Point(xcoord[3],ycoord[3]));
			path.add(new Point(xcoord[4],ycoord[4]));
		}
		// insert the last point.
		path.add(new Point(_targetSlot.getAnchor()));

		return path;
	}

	/**
	 * Moves the current selected intermediate line
	 * of the link according to the new coordinates.
	 * 
	 * @param xycoord the new coordinates.
	 */
	@Override
	public void move( Point xycoord) {
		_userDefined=true;
		if (_selectedLine<1) return;
		if (_polyline.length == 3) {
			if ( (xycoord.y >_polyline[0].y1+GAP) && (xycoord.y <_polyline[_polyline.length-1].y2-GAP)) {
				_polyline[0].y2=_polyline[1].y1=_polyline[1].y2=_polyline[2].y1=xycoord.y;
			}
		} else {
			if (_selectedLine==1) {
				if ( (xycoord.y>_polyline[0].y1-GAP)) {
					_polyline[0].y2=_polyline[1].y1=_polyline[1].y2=_polyline[2].y1=xycoord.y;
				}
			} else if (_selectedLine==2) {
				if ( (xycoord.x >_polyline[0].x1+GAP) && (xycoord.x <_polyline[_polyline.length-1].x2-GAP)) {
					_polyline[1].x2=_polyline[2].x1=_polyline[2].x2=_polyline[3].x1=xycoord.x;
				}
			} else if (_selectedLine==3) {
				if ( (xycoord.y <_polyline[_polyline.length-1].y2-GAP)) {
					_polyline[4].y1=_polyline[3].y1=_polyline[3].y2=_polyline[2].y2=xycoord.y;
				}
			}
		}
	}

	/**
	 * Modifies the link because one the source or target
	 * items has been moved.
	 * @return false if the link cannot be modified because it becomes too complex.
	 */
	@Override
	protected boolean modifyPolyline() {
		double sourceX = _targetSlot.getConnectedSlot().getAnchor().getX();
		double sourceY = _targetSlot.getConnectedSlot().getAnchor().getY();
		double targetX = _targetSlot.getAnchor().getX();
		double targetY = _targetSlot.getAnchor().getY();

		if ((_polyline.length==3 && sourceY >= targetY) || (_polyline.length==5 && sourceY < targetY)) {
			return false;
		}

		if (sourceY != _polyline[0].y1 || sourceX != _polyline[0].x1) {
			_polyline[0].x1 = _polyline[0].x2 = _polyline[1].x1 = (float)sourceX;
			_polyline[0].y1=(float)sourceY;
			if (sourceY>_polyline[0].y2) {
				return false;
			}
		} else if (targetY != _polyline[_polyline.length-1].y1 || targetX != _polyline[_polyline.length-1].x1) {
			_polyline[_polyline.length-1].x2 = _polyline[_polyline.length-1].x1 = _polyline[_polyline.length-2].x2 = (float)targetX;
			_polyline[_polyline.length-1].y2=(float)targetY;
			if (targetY<_polyline[_polyline.length-1].y1) {
				return false;
			}
		}
		return true;
	}
}