/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Document;
import javax.swing.text.Highlighter;
import javax.swing.text.JTextComponent;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.utils.GDialog;

/**
 * The <i>Console</i> is used to display styled text.
 * 
 * @author Régis CLOUARD - Dec 29, 2006
 * @author Régis CLOUARD - Dec 12, 2008 (distinction error message)
 * @author Régis CLOUARD - Jun 30, 2010 (improve rendering)
 * @author Régis CLOUARD - Apr 01, 2011 (handle \n \t, etc in the console + save)
 */
public final class Console extends JTextPane {
	private static final Highlighter _highlighter = new DefaultHighlighter();
	private final WordSearcher _searcher;
	/** Custom output PrintStream. */
	private PrintStream _printOutputStream;
	/** Custom system PrintStream. */
	private PrintStream _printSystemStream;
	/** Custom error PrintStream. */
	private PrintStream _printErrorStream;
	/** The StyleContext for the Console. */
	private ConsoleStyleContext styleContext = new ConsoleStyleContext();

	private ReadWriteThread _outputReadWriteThread;
	private ReadWriteThread _systemReadWriteThread;
	private ReadWriteThread _errorReadWriteThread;

	/**
	 * Creates a new console.
	 */
	public Console( ) {
		setEditable(false);
		_searcher = new WordSearcher(this);

		addMouseListener( new MouseAdapter() {
			private final void showPopupMenu(final MouseEvent event) {
				JPopupMenu _popup = new JPopupMenu();
				JMenuItem erase = new JMenuItem(Ariane.lang.getString("Workspace.console.clear"));
				erase.addActionListener( new ActionListener() {
					@Override
					public final void actionPerformed(ActionEvent e) {
						clear();
					}
				});
				JMenuItem search = new JMenuItem(Ariane.lang.getString("Workspace.console.search"));
				search.addActionListener( new ActionListener() {
					@Override
					public final void actionPerformed(ActionEvent e) {
						search(event.getLocationOnScreen());
					}
				});
				JMenuItem save = new JMenuItem(Ariane.lang.getString("Workspace.console.save"));
				save.addActionListener( new ActionListener() {
					@Override
					public final void actionPerformed(ActionEvent e) {
						save();
					}
				});
				_popup.add(erase);
				_popup.add(search);
				_popup.add(save);
				_popup.show((Component)event.getSource(), event.getX(), event.getY());
			}
			/** Opens the contextual menu */
			@Override
			public void mousePressed(MouseEvent event) {
				if (event.isPopupTrigger()) showPopupMenu(event);
			}
			/** Opens the contextual menu */
			@Override
			public void mouseReleased(MouseEvent event) {
				if (event.isPopupTrigger()) showPopupMenu(event);
			}
		});

		setHighlighter(_highlighter);
		redirect();
	}

	/**
	 * Clears the console.
	 */
	public final void clear( ) {
		StyledDocument document = getStyledDocument();
		try {
			document.remove(0,document.getLength());
		} catch (BadLocationException badLocationException) {
			//			System.err.("Problem with the console");
		}
	}

	/**
	 * Searches a word in the log console.
	 */
	private final void search( Point mouseCoordinates ) {
		final GDialog dialog = new GDialog(Ariane.frame,"Find",false);
		dialog.setLayout(new BorderLayout(1,1));
		dialog.setAlwaysOnTop(true);

		JPanel content = new JPanel();
		final JTextField textfield = new JTextField(10);
		content.add(new JLabel(Ariane.lang.getString("Workspace.console.searchTitle")));
		content.add(textfield);
		dialog.add(content, BorderLayout.CENTER);

		JButton okButton = new JButton(Ariane.lang.getString("Workspace.console.findButton"));
		okButton.addActionListener( new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				String word = textfield.getText().trim();
				int offset = _searcher.search(word);
				if (offset != -1) {
					try {
						Console.this.scrollRectToVisible(Console.this.modelToView(offset));
					} catch ( BadLocationException exception ) {
					}
				}
				dialog.dispose();
			}
		});
		JButton cancelButton = new JButton(Ariane.lang.getString("Workspace.console.closeButton"));
		cancelButton.addActionListener( new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				dialog.dispose();
			}
		});
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		dialog.add(buttonPanel, BorderLayout.SOUTH);
		dialog.pack();
		dialog.setLocation(mouseCoordinates);
		dialog.setVisible(true);
		dialog.setDefaultButton(okButton);
		dialog.setCancelButton(cancelButton);
	}

	/**
	 * Saves the console contents in a file.
	 */
	private final void save( ) {
		String currentDir = Workspace.getCurrentDir(); 
		File currentFileName = new File(currentDir);

		JFileChooser fileChooser = new JFileChooser(currentFileName);
		fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
		fileChooser.setDialogTitle(Ariane.lang.getString("FileChooser.titleSaveLog"));

		if (fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
			File theFile = fileChooser.getSelectedFile();
			if (theFile != null){
				if (theFile.exists())
					if (JOptionPane.showConfirmDialog(Ariane.workspace,
							Ariane.lang.getString("Workspace.dialog.message.part1")
							+theFile.getName()
							+Ariane.lang.getString("Workspace.dialog.message.part2"),
							Ariane.lang.getString("Workspace.dialog.title"),
							JOptionPane.YES_NO_OPTION)==JOptionPane.NO_OPTION) {
						return;
					}
				PrintWriter out=null;
				try {
					String s = getDocument().getText(0, getDocument().getLength());
					out = new PrintWriter(new BufferedWriter(new FileWriter(theFile)));
					out.print(s);
					Ariane.workspace.printSystemMessage(theFile + Ariane.lang.getString("Workspace.saving.success"));
				} catch( Exception ex) {
					Ariane.workspace.printErrorMessage(Ariane.lang.getString("Workspace.saving.error1")+getName()+" "+ex.getLocalizedMessage());
				} finally {
					if (out!=null) out.close();
				}
			}
		}
	}

	/**
	 * Use to search and highlight the word.
	 */
	class WordSearcher {
		protected JTextComponent _comp;
		protected Highlighter.HighlightPainter _painter;

		private WordSearcher( JTextComponent comp ) {
			_comp = comp;
			_painter = new DefaultHighlighter.DefaultHighlightPainter(Color.LIGHT_GRAY);
		}

		/**
		 * Searches for the given word and returns the offset of the
		 * first occurrence. Highlight are added for all occurrence found.
		 */
		private int search( String word ) {
			int firstOffset = -1;

			// remove any existing highlights for last word.
			Highlighter highlighter = _comp.getHighlighter();
			highlighter.removeAllHighlights();
			if (word == null || word.equals("")) {
				return (-1);
			}

			// look for the word - insensitive search
			String content = null;
			try {
				Document d = _comp.getDocument();
				content = d.getText(0,d.getLength()).toLowerCase();
			} catch ( BadLocationException e ) {
				return -1;
			}
			word = word.toLowerCase();
			int lastIndex =0;
			int wordSize = word.length();
			while ( (lastIndex = content.indexOf(word, lastIndex)) != -1) {
				int endIndex = lastIndex + wordSize;
				try {
					highlighter.addHighlight(lastIndex, endIndex, _painter);
				} catch ( BadLocationException e ) {

				}
				if (firstOffset == -1) {
					firstOffset = lastIndex;
				}
				lastIndex = endIndex;
			}
			return firstOffset;
		}
	}

	/**
	 * Displays the specified text in the console.
	 * The text is supposed to be a message from
	 * the execution of the items.
	 * 
	 * @param text the text to be displayed.
	 */
	public final void displayOutputMessage( String text ) {
		final int maxLength=200;
		while (text.length()>maxLength) {
			_printOutputStream.println(text.substring(0, maxLength));
			_printOutputStream.flush();
			synchronized(_outputReadWriteThread) { _outputReadWriteThread.notify(); }
			text = text.substring(maxLength);
		}
		_printOutputStream.println(text);
		_printOutputStream.flush();
		synchronized(_outputReadWriteThread) { _outputReadWriteThread.notify(); }
	}

	/**
	 * Displays the specified text in the console.
	 * The text is supposed to be an error message.
	 * 
	 * @param text the text to be displayed.
	 */
	public final void displayErrorMessage( String text ) {
		_printErrorStream.println(text);
		_printErrorStream.flush();
		synchronized(_errorReadWriteThread) { _errorReadWriteThread.notify(); }
	}

	/**
	 * Displays the specified text in the console.
	 * The text is supposed to be an Ariane message.
	 * 
	 * @param text the text to be displayed.
	 */
	public final void displaySystemMessage( String text ) {
		_printSystemStream.println(text);
		_printSystemStream.flush();
		synchronized(_systemReadWriteThread) { _systemReadWriteThread.notify(); }
	}

	/**
	 * Redirects the flows.
	 */
	private void redirect() {
		try {
			PipedInputStream piCustom = new PipedInputStream();
			PipedOutputStream poCustom = new PipedOutputStream(piCustom);
			_printOutputStream = new PrintStream(poCustom, false);
			_outputReadWriteThread = new ReadWriteThread(piCustom, ConsoleStyleContext.CUSTOM);
			_outputReadWriteThread.start();

			PipedInputStream piOut = new PipedInputStream();
			PipedOutputStream poOut = new PipedOutputStream(piOut);
			_printSystemStream = new PrintStream(poOut, true);
			_systemReadWriteThread = new ReadWriteThread(piOut, ConsoleStyleContext.SYSTEMOUT);
			_systemReadWriteThread.start();

			PipedInputStream piErr = new PipedInputStream();
			PipedOutputStream poErr = new PipedOutputStream(piErr);
			_printErrorStream = new PrintStream(poErr, true);
			_errorReadWriteThread = new ReadWriteThread(piErr, ConsoleStyleContext.SYSTEMERR);
			_errorReadWriteThread.start();
		} catch (IOException e) {
			//			System.err.("Problem with the console.");
		}
	}

	/**
	 * The thread that will write everything to the text area.
	 */
	class ReadWriteThread extends Thread {
		/** The InputStream of this Thread */
		private PipedInputStream _pipedInputStream;
		/** The type (CUSTOM, SYSTEMOUT, SYSTEMERR) of this Thread */
		private String _type;

		/** Create the ReaderThread. */
		ReadWriteThread( PipedInputStream pi, String type ) {
			super();
			_pipedInputStream = pi;
			_type = type;
		}

		@Override
		public void run() {
			final Document doc = Console.this.getDocument();
			final AttributeSet attset = styleContext.getStyle(_type);
			BufferedReader br = new BufferedReader(new InputStreamReader(_pipedInputStream));

			while (true) {
				try {
					String line;
					while (_pipedInputStream.available()>0 ){
						while ((line = br.readLine())!=null) {
							try {
								synchronized (doc) {
									if (doc.getLength()>100000) {
										doc.remove(0, line.length());
									}
									doc.insertString(doc.getLength(), line+"\n", attset);
									Console.this.setCaretPosition(Console.this.getDocument().getLength());
								}
								sleep(50);
							} catch (BadLocationException e) {
//								System.err.("Console: error1");
							}
						}
					}
					synchronized (this) { wait(); }
				} catch (IOException e) {
//					System.err.("Thread: "+this);
//					e.printStackTrace();
				} catch( InterruptedException e) {
//					System.err.("Console: error3");
				}
			}
		}
	}

	/**
	 * The style context defining the styles of each type of PrintStream.
	 */
	final class ConsoleStyleContext extends StyleContext {
		/** The name of the Style used for Custom messages. */
		public static final String CUSTOM = "Custom";
		/** The name of the Style used for system messages. */
		public static final String SYSTEMOUT = "SystemOut";
		/** The name of the Style used for error messages. */
		public static final String SYSTEMERR = "SystemErr";

		/** Creates the style context for the Console. */
		public ConsoleStyleContext() {
			super();
			Style root = getStyle(DEFAULT_STYLE);
			Style s = addStyle(CUSTOM, root);
			StyleConstants.setForeground(s, Color.BLUE);
			s = addStyle(SYSTEMOUT, root);
			StyleConstants.setForeground(s, Color.BLACK);
			s = addStyle(SYSTEMERR, root);
			StyleConstants.setForeground(s, Color.RED);
		}
	}
}