/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.WindowConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTMLEditorKit.InsertHTMLTextAction;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

import fr.greyc.pantheon.ariane.Ariane;

/**
 * <code>HTMLDocumentEditor.java</code> is a dialog window to edit HTML
 * documents.
 * 
 * @author Régis CLOUARD - Jan 3, 2012
 */
public class HTMLDocumentEditor extends JDialog {
	/** Return value if approve (yes, ok) is chosen. */
	public static final int OK_OPTION = JOptionPane.OK_OPTION;

	/** Return value if an error occurred. */
	public static final int NO_OPTION = JOptionPane.NO_OPTION;

	private HTMLDocument _document;
	private JTextPane _htmlEditorPane;
	private JTextArea _textEditorPane;
	private JTabbedPane _tabbedPane;
	private File _currentFile;
	private int _answer;

	/** Listener for the edits on the current document. */
	protected UndoableEditListener _undoHandler = new UndoHandler();

	/** UndoManager that we add edits to. */
	protected UndoManager _undoManager = new UndoManager();

	private UndoAction _undoAction;
	private RedoAction _redoAction;

	public HTMLDocumentEditor(JDialog parent) {
		super(parent, "HTML Editor", true);
		HTMLEditorKit editorKit = new HTMLEditorKit();
		_document = (HTMLDocument) editorKit.createDefaultDocument();
		init();
	}

	/**
	 * Edits the current file.
	 * 
	 * @return YES_OPTION if the current document is to considered, NO_OPTION otherwise.
	 */
	public int edit() {
		try {
			if (_currentFile == null) {
				startNewDocument();
			} else {
				setNewDocument(_currentFile);
			}
			setVisible(true);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		return _answer;
	}

	public File getSelectedFile() {
		return _currentFile;
	}

	public void setSelectedFile(File file) {
		_currentFile = file;
	}

	private void init(){
		ClassLoader classLoader= getClass().getClassLoader();

		JToolBar toolbar1 = new JToolBar();
		toolbar1.setFloatable(false);
		toolbar1.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
		toolbar1.setBorderPainted(true);
		toolbar1.setOpaque(false);

		EditorAction quitA = new EditorAction("Quit", new ImageIcon(classLoader.getResource(Ariane.iconDir+"logout.png")))
		{ @Override
			public void actionPerformed(ActionEvent e) { quitConfirmed(); }
		};
		JToolbarButton quitB=new JToolbarButton(quitA);
		toolbar1.add(quitB);

		toolbar1.addSeparator();

		EditorAction cutAction = new EditorAction("Cut", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/cut.png")),
				new DefaultEditorKit.CutAction());
		EditorAction copyAction = new EditorAction("Copy", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/copy.png")),
				new DefaultEditorKit.CopyAction());
		EditorAction pasteAction = new EditorAction("Paste", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/paste.png")),
				new DefaultEditorKit.PasteAction());
		toolbar1.add(new JToolbarButton(cutAction));
		toolbar1.add(new JToolbarButton(copyAction));
		toolbar1.add(new JToolbarButton(pasteAction));

		toolbar1.addSeparator();

		_undoAction = new UndoAction(new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/undo.png")));
		_redoAction = new RedoAction(new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/redo.png")));
		toolbar1.add(new JToolbarButton(_undoAction));
		toolbar1.add(new JToolbarButton(_redoAction));

		JToolBar toolbar2 = new JToolBar();
		toolbar2.setFloatable(false);
		toolbar2.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
		toolbar2.setBorderPainted(true);
		toolbar2.setOpaque(false);

		ListElement[] blockElements = ListElement.createBlockElementList();
		JComboBox blockElementMenu = new JComboBox(blockElements);
		blockElementMenu.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox)e.getSource();
				ListElement element= (ListElement)cb.getSelectedItem();
				element.actionPerformed(e);
			}
		});
		toolbar2.add(blockElementMenu);

		toolbar2.addSeparator();

		EditorAction boldAction = new EditorAction("Bold", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/bold.png")),
				new StyledEditorKit.BoldAction());
		EditorAction italicAction = new EditorAction("Italic", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/italic.png")),
				new StyledEditorKit.ItalicAction());
		EditorAction underlineAction = new EditorAction("Underline", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/underline.png")),
				new StyledEditorKit.UnderlineAction());
		EditorAction strikeThroughAction= new EditorAction("StrikeThrough", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/strikethrough.png")),
				new StrikeThroughAction());
		EditorAction subscriptAction = new EditorAction("Subscript", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/subscript.png")),
				new SubscriptAction());
		EditorAction superscriptAction = new EditorAction("Superscript", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/superscript.png")),
				new SuperscriptAction());

		toolbar2.add(new JToolbarButton(boldAction));
		toolbar2.add(new JToolbarButton(italicAction));
		toolbar2.add(new JToolbarButton(underlineAction));
		toolbar2.add(new JToolbarButton(strikeThroughAction));
		toolbar2.add(new JToolbarButton(subscriptAction));
		toolbar2.add(new JToolbarButton(superscriptAction));

		toolbar2.addSeparator();

		EditorAction alignLeft = new EditorAction("Alignleft", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/align_left.png")),
				new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_LEFT));
		toolbar2.add(new JToolbarButton(alignLeft));

		EditorAction alignCenter= new EditorAction("AlignCenter", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/align_center.png")),
				new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_CENTER));
		toolbar2.add(new JToolbarButton(alignCenter));

		EditorAction alignRight= new EditorAction("AlignRight", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/align_right.png")),
				new StyledEditorKit.AlignmentAction (null, StyleConstants.ALIGN_RIGHT));
		toolbar2.add(new JToolbarButton(alignRight));

		EditorAction alignJustified = new EditorAction("AlignJustifed", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/align_justified.png")),
				new StyledEditorKit.AlignmentAction (null, StyleConstants.ALIGN_JUSTIFIED));
		toolbar2.add(new JToolbarButton(alignJustified ));

		String[] fontTypes = {"Default", "SansSerif", "Serif", "Monospaced", "Dialog", "DialogInput"};
		JComboBox fontTypeMenu = new JComboBox(fontTypes);
		fontTypeMenu.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox)e.getSource();
				String font = (String)cb.getSelectedItem();
				Action a = new StyledEditorKit.FontFamilyAction(font, font);
				a.actionPerformed(e);
			}
		});

		toolbar2.add(fontTypeMenu);
		Integer[] fontSizes = {6, 8, 10, 12, 14, 16, 20,24, 32, 36, 48, 72 };
		JComboBox fontSizeMenu = new JComboBox(fontSizes);
		fontSizeMenu.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox)e.getSource();
				Integer size = (Integer)cb.getSelectedItem();
				Action a = new StyledEditorKit.FontSizeAction(String.valueOf(size), size);
				a.actionPerformed(e);
			}
		});
		toolbar2.add(fontSizeMenu);
		fontSizeMenu.setSelectedIndex(4);
		Pair[] colors = { new Pair("Red", Color.RED),
				new Pair("Orange", Color.ORANGE),
				new Pair("Yellow", Color.YELLOW), 
				new Pair("Green", Color.GREEN),
				new Pair("Blue", Color.BLUE),
				new Pair("Cyan", Color.CYAN),
				new Pair("Magenta", Color.MAGENTA),
				new Pair("Black", Color.BLACK)};
		JComboBox colorBox = new JComboBox(colors);
		colorBox.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox)e.getSource();
				Pair pair = (Pair)cb.getSelectedItem();
				Action a = new StyledEditorKit.ForegroundAction(pair.name, pair.color);
				a.actionPerformed(e);
			}
		});
		toolbar2.add(colorBox);

		EditorAction breakRule= new EditorAction("InsertBreak", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/br.png")),
				new HTMLEditorKit.InsertHTMLTextAction("InsertBreak", "<br>", HTML.Tag.BODY, HTML.Tag.BR));
		toolbar2.add(new JToolbarButton(breakRule));

		EditorAction hRule= new EditorAction("InsertHR", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/hrule.png")),
				new HTMLEditorKit.InsertHTMLTextAction("InsertHR", "<hr>", HTML.Tag.BODY, HTML.Tag.HR));
		toolbar2.add(new JToolbarButton(hRule));

		EditorAction unorderedList= new EditorAction("UnorderedList", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/listunordered.png")),
				new InsertHTMLTextAction("Itemize", "<ul><li></li></ul>",
						HTML.Tag.UL, HTML.Tag.LI,
						HTML.Tag.BODY, HTML.Tag.UL));
		toolbar2.add(new JToolbarButton(unorderedList));

		EditorAction orderedList= new EditorAction("OrderedList", new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/listordered.png")),
				new InsertHTMLTextAction("Enumerate", "<ol><li></li></ol>",
				HTML.Tag.UL, HTML.Tag.LI,
				HTML.Tag.BODY, HTML.Tag.OL));
		toolbar2.add(new JToolbarButton(orderedList));

		toolbar2.add(new JToolbarButton(new LinkAction(new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/link.png")))));
		toolbar2.add(new JToolbarButton(new ImageAction(new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/image.png")))));
		toolbar2.add(new JToolbarButton(new TableAction(new ImageIcon(classLoader.getResource(Ariane.iconDir+"html/table.png")))));

		toolbar2.addSeparator();

		_document.addUndoableEditListener(_undoHandler);
		resetUndoManager();

		/////
		/* html and text views each have a tab */
		_tabbedPane = new JTabbedPane();

		/* create scrolling html viewer and add to tabbed pane */
		_htmlEditorPane = new JTextPane(_document);
		_htmlEditorPane.setContentType("text/html; charset=EUC-JP");
		JScrollPane htmlEditorScrollPane = new JScrollPane(_htmlEditorPane);
		htmlEditorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		htmlEditorScrollPane.setPreferredSize(new Dimension(500, 300));
		_tabbedPane.addTab("HTML", null, htmlEditorScrollPane, "HTML View");

		/* create scrolling text viewer and add to tabbed pane */
		_textEditorPane = new JTextArea();
		JScrollPane textEditorScrollPane = new JScrollPane(_textEditorPane);
		textEditorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		textEditorScrollPane.setPreferredSize(new Dimension(500, 300));
		_tabbedPane.addTab("TEXT", null, textEditorScrollPane, "Text View");

		_tabbedPane.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				updateEditView();
			}
		});

		/////
		JPanel toolbarPanel = new JPanel();
		toolbarPanel.setLayout(new BorderLayout());
		toolbarPanel.add(toolbar1, BorderLayout.NORTH);
		toolbarPanel.add(toolbar2, BorderLayout.CENTER);

		getContentPane().add(toolbarPanel, BorderLayout.NORTH);
		getContentPane().add(_tabbedPane, BorderLayout.CENTER);

		pack();
		setLocationRelativeTo(getParent());
		startNewDocument();
		addWindowListener(new FrameListener());
	}

	private void updateEditView() {
		if (_tabbedPane.getSelectedIndex() == 0) {
			_htmlEditorPane.setText("");
			insertHTML(_htmlEditorPane, _textEditorPane.getText(), 0);
			// _htmlEditorPane.setCaretPosition(_textEditorPane.getCaretPosition());
		} else {
//			String t = deIndent(_htmlEditorPane.getText());
			_textEditorPane.setText(_htmlEditorPane.getText());
			// _textEditorPane.setCaretPosition(_htmlEditorPane.getCaretPosition());
		}
	}

	private void insertHTML(JEditorPane editor, String html, int location) {
		try {
			HTMLEditorKit kit = (HTMLEditorKit) editor.getEditorKit();
			Document doc = editor.getDocument();
			StringReader reader = new StringReader(html);
			kit.read(reader, doc, location);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void resetUndoManager() {
		_undoManager.discardAllEdits();
		_undoAction.update();
		_redoAction.update();
	}

	private void startNewDocument() {
		Document oldDoc = _htmlEditorPane.getDocument();
		if (oldDoc != null) {
			oldDoc.removeUndoableEditListener(_undoHandler);
		}
		HTMLEditorKit editorKit = new HTMLEditorKit();
		_document = (HTMLDocument) editorKit.createDefaultDocument();
		_htmlEditorPane.setDocument(_document);
		_currentFile = null;
		setTitle("HTMLDocumentEditor");
		_htmlEditorPane.getDocument().addUndoableEditListener(_undoHandler);
		resetUndoManager();
	}

	private void setNewDocument(File file) throws IOException,
	BadLocationException {
		// Remove file:
		_currentFile = file;
		setTitle(_currentFile.getName());
		FileReader fr=null;
		try {
			fr = new FileReader(_currentFile);
		} catch ( FileNotFoundException e) {
			startNewDocument();
			return;
		}
		Document oldDoc = _htmlEditorPane.getDocument();
		if (oldDoc != null) {
			oldDoc.removeUndoableEditListener(_undoHandler);
		}
		HTMLEditorKit editorKit = new HTMLEditorKit();
		_document = (HTMLDocument) editorKit.createDefaultDocument();
		_document.putProperty("IgnoreCharsetDirective", new Boolean(true));
		try {
			editorKit.read(fr, _document, 0);
		} catch (Exception e) {
			e.printStackTrace();
		}
		_document.addUndoableEditListener(_undoHandler);
		_htmlEditorPane.setDocument(_document);
		resetUndoManager();
	}

	private void saveDocument() {
		if (_currentFile != null) {
			try {
				FileWriter fw = new FileWriter(_currentFile);
				fw.write(_htmlEditorPane.getText());
				fw.close();
			} catch (FileNotFoundException fnfe) {
				System.err.println("FileNotFoundException: "
						+ fnfe.getMessage());
			} catch (IOException ioe) {
				System.err.println("IOException: " + ioe.getMessage());
			}
		} else {
			saveDocumentAs();
		}
	}

	private void saveDocumentAs() {
		try {
			File current = new File(".");
			JFileChooser chooser = new JFileChooser(current);
			chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			chooser.setFileFilter(new HTMLFileFilter());
			int approval = chooser.showSaveDialog(this);
			if (approval == JFileChooser.APPROVE_OPTION) {
				File newFile = chooser.getSelectedFile();
				if (newFile.exists()) {
					String message = newFile.getAbsolutePath()
					+ Ariane.lang.getString("HTMLDocumentEditor.overrideMessage");
					if (JOptionPane.showConfirmDialog(this, message) == JOptionPane.YES_OPTION) {
						_currentFile = newFile;
						setTitle(_currentFile.getName());
						FileWriter fw = new FileWriter(_currentFile);
						fw.write(_htmlEditorPane.getText());
						fw.close();
//						System.out.("Saved "+ _currentFile.getAbsolutePath());
					}
				} else {
					_currentFile = new File(newFile.getAbsolutePath());
					setTitle(_currentFile.getName());
					FileWriter fw = new FileWriter(_currentFile);
					fw.write(_htmlEditorPane.getText());
					fw.close();
//					System.out.("Saved " + _currentFile.getAbsolutePath());
				}
			}
		} catch (FileNotFoundException fnfe) {
			System.err.println("FileNotFoundException: " + fnfe.getMessage());
		} catch (IOException ioe) {
			System.err.println("IOException: " + ioe.getMessage());
		}
	}

	private boolean quitConfirmed() {
		_answer = JOptionPane.showConfirmDialog(this,
				Ariane.lang.getString("HTMLDocumentEditor.quitMessage"));
		if (_answer == JOptionPane.CANCEL_OPTION) {
			return false;
		} else if (_answer == JOptionPane.OK_OPTION) {
			saveDocument();
		}
		setVisible(false);
		return true;
	}

	class FrameListener extends WindowAdapter {
		@Override
		public void windowClosing(WindowEvent we) {
			if (quitConfirmed()) {
				((JDialog) (we.getWindow()))
				.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			} else {
				((JDialog) (we.getWindow()))
				.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			}
		}
	}

	class SubscriptAction extends StyledEditorKit.StyledTextAction {

		public SubscriptAction() {
			super(StyleConstants.Subscript.toString());
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			JEditorPane editor = getEditor(ae);
			if (editor != null) {
				StyledEditorKit kit = getStyledEditorKit(editor);
				MutableAttributeSet attr = kit.getInputAttributes();
				boolean subscript = (StyleConstants.isSubscript(attr)) ? false : true;
				SimpleAttributeSet sas = new SimpleAttributeSet();
				StyleConstants.setSubscript(sas, subscript);
				setCharacterAttributes(editor, sas, false);
			}
		}
	}

	class SuperscriptAction extends StyledEditorKit.StyledTextAction {

		public SuperscriptAction() {
			super(StyleConstants.Superscript.toString());
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			JEditorPane editor = getEditor(ae);
			if (editor != null) {
				StyledEditorKit kit = getStyledEditorKit(editor);
				MutableAttributeSet attr = kit.getInputAttributes();
				boolean superscript = (StyleConstants.isSuperscript(attr)) ? false : true;
				SimpleAttributeSet sas = new SimpleAttributeSet();
				StyleConstants.setSuperscript(sas, superscript);
				setCharacterAttributes(editor, sas, false);
			}
		}
	}

	class StrikeThroughAction extends StyledEditorKit.StyledTextAction {

		public StrikeThroughAction() {
			super(StyleConstants.StrikeThrough.toString());
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			JEditorPane editor = getEditor(ae);
			if (editor != null) {
				StyledEditorKit kit = getStyledEditorKit(editor);
				MutableAttributeSet attr = kit.getInputAttributes();
				boolean strikeThrough = (StyleConstants.isStrikeThrough(attr)) ? false
						: true;
				SimpleAttributeSet sas = new SimpleAttributeSet();
				StyleConstants.setStrikeThrough(sas, strikeThrough);
				setCharacterAttributes(editor, sas, false);
			}
		}
	}

	class HTMLFileFilter extends javax.swing.filechooser.FileFilter {
		@Override
		public boolean accept(File f) {
			return ((f.isDirectory()) || (f.getName().toLowerCase()
					.indexOf(".htm") > 0));
		}

		@Override
		public String getDescription() {
			return "html";
		}
	}

	class UndoHandler implements UndoableEditListener {
		/**
		 * Messaged when the Document has created an edit, the edit is added to
		 * <code>undo</code>, an instance of UndoManager.
		 */
		@Override
		public void undoableEditHappened(UndoableEditEvent e) {
			_undoManager.addEdit(e.getEdit());
			_undoAction.update();
			_redoAction.update();
		}
	}

	class UndoAction extends AbstractAction {
		public UndoAction( Icon icon) {
			super("Undo", icon);
			setEnabled(false);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			try {
				_undoManager.undo();
			} catch (CannotUndoException ex) {
//				System.out.println("Unable to undo: " + ex);
				// ex.printStackTrace();
			}
			update();
			_redoAction.update();
		}

		protected void update() {
			if (_undoManager.canUndo()) {
				setEnabled(true);
				putValue(Action.NAME, _undoManager.getUndoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, "Undo");
			}
		}
	}

	class RedoAction extends AbstractAction {

		public RedoAction( Icon icon ) {
			super("Redo", icon);
			setEnabled(false);
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			try {
				_undoManager.redo();
			} catch (CannotRedoException ex) {
				System.err.println("Unable to redo: " + ex);
				ex.printStackTrace();
			}
			update();
			_undoAction.update();
		}

		protected void update() {
			if (_undoManager.canRedo()) {
				setEnabled(true);
				putValue(Action.NAME, _undoManager.getRedoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, "Redo");
			}
		}
	}

	class Pair {
		String name;
		Color color;

		public Pair(String name, Color color) {
			this.name = name;
			this.color = color;
		}

		@Override
		public String toString() {
			return name;
		}
	}

	static class ListElement extends HTMLEditorKit.InsertHTMLTextAction {
		public ListElement(String name, String html, HTML.Tag parentTag, HTML.Tag addTag) {
			super(name, html, parentTag, addTag, null, null);
		}

		public static ListElement[] createBlockElementList() {
			ListElement[] listElements = {
					new ListElement("Body Text", "<div> </div>", HTML.Tag.BODY, HTML.Tag.DIV),
					new ListElement("Paragraph", "<p> </p>", HTML.Tag.BODY, HTML.Tag.P),
					new ListElement("Preformatted", "<pre> </pre>", HTML.Tag.BODY, HTML.Tag.PRE),
					new ListElement("Heading 1", "<h1> </h1>", HTML.Tag.BODY, HTML.Tag.H1),
					new ListElement("Heading 2", "<h2> </h2>", HTML.Tag.BODY, HTML.Tag.H2),
					new ListElement("Heading 3", "<h3> </h3>", HTML.Tag.BODY, HTML.Tag.H3),
					new ListElement("Heading 4", "<h4> </h4>", HTML.Tag.BODY, HTML.Tag.H4),
					new ListElement("Heading 5", "<h5> </h5>", HTML.Tag.BODY, HTML.Tag.H5),
					new ListElement("Heading 6", "<h6> </h6>", HTML.Tag.BODY, HTML.Tag.H6) };
			return listElements;
		}

		@Override
		public String toString() {
			return (String) getValue(Action.NAME);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			super.actionPerformed(ae);
			JEditorPane e = getEditor(ae);
			e.requestFocusInWindow();
		}
	}

	public class TableAction extends EditorAction {
		public TableAction( Icon icon) {
			super("InsertTable", icon);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			JPanel panel = new JPanel(new GridLayout(2,2));
			JSpinner rowEditor = new JSpinner(new SpinnerNumberModel(1,1,10,1));
			JSpinner colEditor = new JSpinner(new SpinnerNumberModel(1,1,10,1));
			panel.add(new JLabel("Number of columns"));
			panel.add(colEditor);
			panel.add(new JLabel("Number of rows"));
			panel.add(rowEditor);

			if (JOptionPane.showConfirmDialog(HTMLDocumentEditor.this, panel, "Table", JOptionPane.OK_CANCEL_OPTION)== JOptionPane.OK_OPTION) {
				HTMLEditorKit kit = (HTMLEditorKit)_htmlEditorPane.getEditorKit();
				HTMLDocument doc = (HTMLDocument)_htmlEditorPane.getDocument();
				Integer numRows = (Integer)rowEditor.getValue();
				Integer numCols = (Integer)colEditor.getValue();

				String html = "<table border=1>";
				for(int row = 1; row <= numRows; row++) {
					html += "<tr>\n"; 
					for(int col = 1; col <= numCols; col++) {
						html += "\t<td>\n</td>\n";
					}
					html += "</tr>\n";
				}
				html += "</table>";
				try {
					kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), html, 0, 0, HTML.Tag.TABLE);
				} catch (Exception e) {
					JOptionPane.showMessageDialog(HTMLDocumentEditor.this, "Image Not Loaded", "ERROR", JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
				}
			}
		}
	}

	public class LinkAction extends EditorAction {
		public LinkAction( Icon icon) {
			super("InsertURL", icon);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			HTMLEditorKit kit = (HTMLEditorKit)_htmlEditorPane.getEditorKit();
			HTMLDocument doc = (HTMLDocument)_htmlEditorPane.getDocument();
			String value = JOptionPane.showInputDialog(HTMLDocumentEditor.this, "URL:");
			try {
				kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), "<a href=\""+ value + "\">", 0, 0, HTML.Tag.A);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(HTMLDocumentEditor.this,
						"Image Not Loaded", "ERROR", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
			_htmlEditorPane.requestFocusInWindow();
		}
	}

	public class ImageAction extends EditorAction {
		public ImageAction( Icon icon) {
			super("InsertImage", icon);
		}

		@Override
		public void actionPerformed(ActionEvent ae) {
			HTMLEditorKit kit = (HTMLEditorKit)_htmlEditorPane.getEditorKit();
			HTMLDocument doc = (HTMLDocument) _htmlEditorPane.getDocument();
			String value = JOptionPane.showInputDialog(HTMLDocumentEditor.this, "Image file:");
			try {
				kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), "<img src=\"" + value + "\">", 0, 0, HTML.Tag.IMG);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(HTMLDocumentEditor.this, "Image Not Loaded", "ERROR", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
		}
	}

	private class EditorAction extends AbstractAction {
		private Action _action;

		public EditorAction( String name, Icon icon ) {
			super(name, icon);
			_action = null;
		}

		public EditorAction( String name, Icon icon, Action action) {
			super(name, icon);
			_action = action;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			_action.actionPerformed(e);
			_htmlEditorPane.requestFocusInWindow();
		}
	}

	class JToolbarButton extends JButton {
		public JToolbarButton( Action action ) {
			super(action);
			setText(null);
			setToolTipText((String)action.getValue(Action.NAME));
		}
	}

	public static void main(String[] args) {
		new HTMLDocumentEditor(null).edit();
	}
}