/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.utils;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

/**
 * The class <code>ImageDecoder</code> provides facilities
 * to decode image files.
 * Recognized image formats are gif, png, jpg and Pandore.
 * ?? tif, bmp, ppm, pgm, pbm
 * 
 * @author Régis CLOUARD - Jan 11, 2007
 */
public class ImageDecoder {
	/**
	 * Decodes the image in the given filename and return a thumbnail
	 * of the image with the specified maximum size.
	 * 
	 * @param filename the filename.
	 * @param maxSize the maximum size both in width and height.
	 * @return the image icon or null of any.
	 */
	public static ImageIcon getThumbnail( String filename, int maxSize ) {
		Image image= null;

		// may be a regular image file (bmp, png, ...)
		try {
			image = ImageIO.read(new File(filename));
		} catch (IOException e) {
		}

		if (image==null) {
			// May be a Pandore image??
			PandoreImageDecoder pan = new PandoreImageDecoder(maxSize+10);
			pan.readImage(filename);
			image = Toolkit.getDefaultToolkit().createImage(pan);
		}

		// Not a regular image
		if (image==null) {
			return null;
		}

		ImageIcon imageIcon = new ImageIcon(image);

		// Resize image.
		if (imageIcon.getIconWidth()>imageIcon.getIconHeight()) {
			if (imageIcon.getIconWidth()>maxSize) {
				return new ImageIcon(image.getScaledInstance(maxSize,-1,Image.SCALE_DEFAULT));
			} else {
				return imageIcon;
			}
		} else {
			if (imageIcon.getIconHeight()>maxSize) {
				return new ImageIcon(image.getScaledInstance(-1,maxSize,Image.SCALE_DEFAULT));
			} else {
				return imageIcon;
			}
		}
	}
}