/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.utils;

import javax.swing.tree.DefaultMutableTreeNode;

/**
 * The class <code>MutableTreeItem</code> defines a 
 * node or a leaf in the tree.
 * @see DefaultMutableTreeNode
 *
 * @author Régis CLOUARD - Nov 20, 2001
 */
public class MutableTreeItem extends DefaultMutableTreeNode {

	/**
	 * Creates a new item that is a node or a leaf.
	 * 
	 * @param object the content of the item.
	 * @param isNode true if the item must be a node.
	 */
	public MutableTreeItem( Object object, boolean isNode ) {
		super(object, isNode);
	}

	/**
	 * Overrides the method isLeaf to set that a node
	 * is a leaf only if it is at the level 3.
	 * 
	 * @return true if it is a leaf.
	 */
	@Override
	public final boolean isLeaf() {
		return (!getAllowsChildren());
	}

	/**
	 * Returns the child that holds the specified name
	 * if exits.
	 * 
	 * @param name the name of the child.
	 * @return the node or null.
	 */
	public final MutableTreeItem getNode( String name ) {
		// Search if the category already exist.
		for (int j=0; j< getChildCount(); j++){
			MutableTreeItem x=(MutableTreeItem)getChildAt(j);
			String xn=x.toString();
			if (name.equalsIgnoreCase(xn)) {
				return x;
			}
		}
		return null;
	}

	/**
	 * Clones recursively the tree.
	 * 
	 * @return the tree to be cloned.
	 */
	@Override
	public final Object clone() {
		MutableTreeItem root=(MutableTreeItem)super.clone();
		for (int i=0; i<getChildCount(); i++)
			root.insert((MutableTreeItem)(((MutableTreeItem)getChildAt(i)).clone()),i);
		return root;
	}
}